from psycopg2 import sql
from ..DataStructures.ObjEvmTokenHistory import ObjEvmTokenHistory
from ..rupine_db import herokuDbAccess

def ParseObjEvmTokenHistory(data):
    retObj = ObjEvmTokenHistory()
    retObj.token_address = data[0]
    retObj.abi = data[1]
    retObj.chain_id = data[2] 
    retObj.symbol = data[3]
    retObj.name = data[4]
    retObj.decimals = data[5]
    retObj.token_class = data[6]
    retObj.totalsupply = data[7]
    retObj.keywords = data[8]
    retObj.telegram_link = data[9] 
    retObj.creator_address = data[10] 
    retObj.creation_timestamp = data[11] 
    retObj.creation_timestamp = data[12] 
    retObj.creation_tx_hash = data[13] 
    retObj.created_at = data[14] 
    retObj.modified_at = data[15] 
    return retObj

#TODO: continue here... implement connection to that table

def postEvmTokenHistory(connection, schema:str, token:ObjEvmTokenHistory):

    query = sql.SQL('INSERT INTO {}.evm_token_history ("timestamp", token_address, chain_id, max_tx_amount_percent, max_wallet_size_percent, is_verified, is_honeypot, buy_tax, sell_tax) \
           VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s)').format(sql.Identifier(schema))
    params = (
        token.timestamp,
        token.token_address,
        token.chain_id,
        token.max_tx_amount_percent,
        token.max_wallet_size_percent,
        token.is_verified,
        token.is_honeypot,
        token.buy_tax,
        token.sell_tax)
    result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
    return result

def getEvmTokenHistory(connection, schema, chain_id, token_address):
    
    # query database    
    query = sql.SQL('SELECT "timestamp", token_address, chain_id, max_tx_amount_percent, max_wallet_size_percent, is_verified, is_honeypot, buy_tax, sell_tax, created_at, modified_at \
        FROM {}.evm_token_history WHERE chain_id = %s AND token_address = %s').format(sql.Identifier(schema))
    result = herokuDbAccess.fetchDataInDatabase(query, [chain_id,token_address], connection)    
    
    # parse into objects
    rows = []
    for tok in result:
        addRow = ParseObjEvmTokenHistory(tok)
        rows.append(addRow)

    # return objects
    return rows


from RupineHeroku.rupine_db import herokuDbAccess
from psycopg2 import sql
from datetime import datetime
import pytz

def postAccountAddress(connection, schema, data):
    query = sql.SQL("INSERT INTO {}.dwh_address_account (chain_id,chain,account,public_address,context) VALUES (%s,%s,%s,%s,%s)").format(sql.Identifier(schema))
    params = (
        data['chain_id'],
        data['chain'],
        data['account'],
        data['public_address'],
        data['context'])
    result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
    return result

def getAccountAddress(connection, schema, account=None, public_address=None, chain=None, chain_id=None):
    conditions = ""
    params = []
    if chain is None and chain_id is None:
        return []
    elif chain is not None:
        conditions = conditions + " AND t.chain = %s"
        params.append(chain)
    elif chain_id is not None:
        conditions = conditions + " AND t.chain_id = %s"
        params.append(chain_id)
    if account is not None:
        query = sql.SQL("SELECT account,public_address,context from {0}.dwh_address_account t WHERE 1=1 %s  AND account = %%s " % (conditions)).format(sql.Identifier(schema))
        params.append(account)
    elif public_address is not None:
        query = sql.SQL("SELECT account,public_address,context from {0}.dwh_address_account t WHERE 1=1 %s AND public_address = %%s" % (conditions)).format(sql.Identifier(schema))
        params.append(public_address)
    else:
        return []

    result = herokuDbAccess.fetchDataInDatabase(query, [*params], connection)  
    return result


# import os
# from dotenv import load_dotenv
# import herokuDbAccess as db
# from datetime import datetime
# load_dotenv()

# if __name__ == '__main__':
#     connection = db.connect(
#         os.environ.get("HEROKU_DB_USER"),
#         os.environ.get("HEROKU_DB_PW"),
#         os.environ.get("HEROKU_DB_HOST"),
#         os.environ.get("HEROKU_DB_PORT"),
#         os.environ.get("HEROKU_DB_DATABASE")
#     )
#     data = {
#         'chain_id':None,
#         'chain':'DeFiChain',
#         'account':'Benni',
#         'public_address':'df1qgssyvcqld7cwxzvnajyt3xlqae67muxrqh06ct',
#         'context':'private'
        
#     }
#     postAccountAddress(connection,'stage',data)
#     print(getAccountAddress(connection,'stage','Benni',chain='DeFiChain'))
    # data = {
    #     'chain_id':1,
    #     'chain':'ETH',
    #     'account':'MYACC',
    #     'address':'dfi1',
    #     'token':'BLUE1',
    #     'buy_timestamp':127897934,
    #     'buy_price':12.4,
    #     'buy_transaction_hashes':'sjdlfjskljdfklsj,jhghjhghj',
    #     'sell_timestamp':893434,
    #     'sell_price':34.35,
    #     'sell_transaction_hashes':'shdsdfjkhsdjkf2,sdjfklj',
    #     'amount':12.2
    # }
    #res = getTaxTransactionRewards(connection,'stage',['dfi1','df1qgssyvcqld7cwxzvnajyt3xlqae67muxrqh06ct'],'DeFiChain',None,'DFI',0)
    #res = getTaxTokens(connection,'stage',['dfi1','df1qgssyvcqld7cwxzvnajyt3xlqae67muxrqh06ct'],'DeFiChain',None)
    # res = getLatestWarehouseDate(connection,'stage','BLUE12','MYACC','Defi',None)
    # print(res)


from psycopg2 import sql
from ..rupine_db import herokuDbAccess

def getCredential(connection, schema, chain_id):
    
    username = getCredential(connection, schema, 'ankr_prod', 'USERNAME', chain_id)
    password = getCredential(connection, schema, 'ankr_prod', 'PASSWORD', chain_id)

    return ':'.join([username,password])

def getCredential(connection, schema, credential_name, credential_type, chain_id):
    
    query = sql.SQL("SELECT credentials_value \
           FROM {}.credentials \
           WHERE credentials_name = %s AND  credentials_type = %s AND chain_id = %s").format(sql.Identifier(schema))
    
    params = (credential_name, credential_type, chain_id)
    
    retVal = herokuDbAccess.fetchDataInDatabase(query, params, connection)
    return retVal[0][0]
    



class ObjEvmTokenHistory:
    timestamp = 0
    token_address = None
    chain_id = None 
    max_tx_amount_percent = None
    max_wallet_size_percent = None
    is_verified = None
    is_honeypot = None
    buy_tax = None
    sell_tax = None
    created_at = 0
    modified_at = 0

    def __init__(self):
        pass

    def __eq__(self, other): 
        if not isinstance(other, ObjEvmTokenHistory):
            # don't attempt to compare against unrelated types
            return NotImplemented

        return (self.timestamp == other.timestamp and 
                self.token_address == other.token_address and 
                self.chain_id == other.chain_id and
                self.max_tx_amount_percent == other.max_tx_amount_percent and
                self.max_wallet_size_percent == other.max_wallet_size_percent and
                self.is_verified == other.is_verified and
                self.is_honeypot == other.is_honeypot and
                self.buy_tax == other.buy_tax and
                self.sell_tax == other.sell_tax and
                self.created_at == other.created_at and
                self.modified_at == other.modified_at)

    def clone(self):
        retTok = ObjEvmTokenHistory()
        retTok.timestamp = self.timestamp
        retTok.token_address = self.token_address
        retTok.chain_id = self.chain_id
        retTok.max_tx_amount_percent = self.max_tx_amount_percent
        retTok.max_wallet_size_percent = self.max_wallet_size_percent
        retTok.is_verified = self.is_verified
        retTok.is_honeypot = self.is_honeypot
        retTok.buy_tax = self.buy_tax
        retTok.sell_tax = self.sell_tax
        retTok.created_at = self.created_at
        retTok.modified_at = self.modified_at
        return retTok


from psycopg2 import sql
from ..DataStructures.ObjEvmToken import ObjEvmToken
from . import herokuDbAccess

class ObjDmhNetwork:
    gecko_id = ''
    gecko_name = ''
    gecko_symbol = ''
    cmc_id = ''
    chain_id = ''
    twitter_name = ''
    twitter_id = ''
    twitter_hashtag = ''
    github_org = ''
    telegram_link = ''
    created_at = 0 
    modified_at= 0

def ParseDataIntoObj(data):
    retObj = ObjDmhNetwork()
    retObj.gecko_id = data[0]
    retObj.gecko_name = data[1]
    retObj.gecko_symbol = data[2]
    retObj.cmc_id = data[3]
    retObj.chain_id = data[4]
    retObj.twitter_name = data[5]
    retObj.twitter_id = data[6]
    retObj.twitter_hashtag = data[7]
    retObj.github_org = data[8]
    retObj.telegram_link = data[9]
    retObj.created_at = data[10]
    retObj.modified_at= data[11]
    return retObj

def getNetworkList(connection, schema):
    
    # query database    
    query = sql.SQL("SELECT gecko_id, gecko_name, gecko_symbol, cmc_id, chain_id, twitter_name, twitter_id, twitter_hashtag, github_org, telegram_link, created_at, modified_at \
        FROM {}.dwh_network").format(sql.Identifier(schema))
    result = herokuDbAccess.fetchDataInDatabase(query, [], connection)    
    
    # parse into objects
    rows = []
    for tok in result:
        addRow = ParseDataIntoObj(tok)
        rows.append(addRow)

    # return objects
    return rows


from RupineHeroku.rupine_db import herokuDbAccess
from psycopg2 import sql
from datetime import datetime
import pytz

def postTaxTransaction(connection, schema, data):

    query = sql.SQL("INSERT INTO {}.tax_transaction (tx_no,chain_id,chain,public_address,timestamp,block_number,transaction_hash,category,token,amount,usd_value,eur_value,fee_token,fee_amount,fee_usd_value,fee_eur_value) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)").format(sql.Identifier(schema))
    params = (
        data['tx_no'],
        data['chain_id'],
        data['chain'],
        data['public_address'],
        data['timestamp'],
        data['block_number'],
        data['transaction_hash'],
        data['category'],
        data['token'],
        data['amount'],
        data['usd_value'],
        data['eur_value'],
        data['fee_token'],
        data['fee_amount'],
        data['fee_usd_value'],
        data['fee_eur_value'])
    result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
    return result

def getTaxTransactionRewards(connection, schema, addresses:list=None, chain=None, chain_id=None, token:str=None, timestamp:int=0, posNegAll:str='all'):
    '''
    Parameters:
        - address: relevant public keys (optional)
        - chain/chain_id: either choose a specifier (necessary)
        - token: String of Token, e.g. URHT, DUSD-URTH, etc. Default is None
        - timestamp: all data with timestamp gte. Default is 0
        - posNegAll: "positive", "negative" or "all". is Amount gte 0, lt 0 or everything. Default is 'all'
    '''
    conditions = ""
    params = []
    if chain is None and chain_id is None:
        return []
    elif chain is not None:
        conditions = conditions + " AND t.chain = %s"
        params.append(chain)
    elif chain_id is not None:
        conditions = conditions + " AND t.chain_id = %s"
        params.append(chain_id)
    
    if addresses is not None:
        vars = ["%s"]*len(addresses)
        conditions = conditions + " AND t.public_address IN ({})".format(','.join(vars))
        params.extend(addresses)

    if token != None:
        conditions = conditions + " AND t.token = %s"
        params.append(token)
    
    if posNegAll == 'positive':
        conditions = conditions + " AND t.amount >= 0"
    elif posNegAll == 'negative':
        conditions = conditions + " AND t.amount < 0"

    query = sql.SQL("SELECT basis.* FROM ( \
                        SELECT t.id,t.chain_id,t.chain,t.public_address,t.timestamp,t.block_number,t.transaction_hash,t.category,t.token,t.amount,t.usd_value,t.eur_value,t.fee_usd_value,t.fee_eur_value, \
                        CASE WHEN c.sort_no IS NULL THEN '999' ELSE c.sort_no END AS sort_no \
                        FROM {0}.tax_transaction t LEFT JOIN {0}.tax_category c ON t.category = c.category \
                        WHERE 1=1 AND t.timestamp >= %%s %s \
                            UNION ALL	\
                        SELECT t.id,t.chain_id,t.chain,t.public_address,t.timestamp,NULL AS block_number,NULL AS transaction_hash,t.category,t.token,t.amount,t.usd_value,t.eur_value,NULL AS fee_usd_value,NULL AS fee_eur_value, \
                        CASE WHEN c.sort_no IS NULL THEN '999' ELSE c.sort_no END AS sort_no \
                        FROM {0}.tax_reward t LEFT JOIN {0}.tax_category c ON t.category = c.category \
                        WHERE 1=1 AND t.timestamp >= %%s %s \
                    ) basis ORDER BY basis.timestamp ASC, basis.sort_no ASC" % (conditions,conditions)).format(sql.Identifier(schema))

    result = herokuDbAccess.fetchDataInDatabase(query, [timestamp,*params,timestamp,*params], connection)    
       
    return result

def getTaxTokens(connection, schema, addresses:list=None, chain=None, chain_id=None):
    '''
    Parameters:
        - address: relevant public keys (optional)
        - chain/chain_id: either choose a specifier (necessary)
        - token: String of Token, e.g. URHT, DUSD-URTH, etc. Default is None
        - timestamp: all data with timestamp gte. Default is 0
        - posNegAll: "positive", "negative" or "all". is Amount gte 0, lt 0 or everything. Default is 'all'
    '''
    conditions = ""
    params = []
    if chain is None and chain_id is None:
        return []
    elif chain is not None:
        conditions = conditions + " AND t.chain = %s"
        params.append(chain)
    elif chain_id is not None:
        conditions = conditions + " AND t.chain_id = %s"
        params.append(chain_id)
    
    if addresses is not None:
        vars = ["%s"]*len(addresses)
        conditions = conditions + " AND t.public_address IN ({})".format(','.join(vars))
        params.extend(addresses)

    query = sql.SQL("SELECT DISTINCT t.token from {0}.tax_transaction t \
                    WHERE 1=1 %s \
                    UNION \
                    SELECT DISTINCT t.token from {0}.tax_reward t \
                    WHERE 1=1 %s" % (conditions,conditions)).format(sql.Identifier(schema))

    result = herokuDbAccess.fetchDataInDatabase(query, [*params,*params], connection)  
    return result

def getLatestWarehouseTxDate(connection, schema, token:str, account:str, chain=None, chain_id=None):
    conditions = ""
    params = []
    if chain is None and chain_id is None:
        return []
    elif chain is not None:
        conditions = conditions + " AND chain = %s"
        params.append(chain)
    elif chain_id is not None:
        conditions = conditions + " AND chain_id = %s"
        params.append(chain_id)

    query = sql.SQL("SELECT t.timestamp,b.latest_rest_tx_no,b.latest_rest_amount FROM {0}.tax_transaction t RIGHT JOIN ( \
                    SELECT DISTINCT FIRST_VALUE(rest_tx_no) OVER (ORDER BY day DESC) as latest_rest_tx_no,FIRST_VALUE(rest_amount) OVER (ORDER BY day desc) as latest_rest_amount FROM {0}.tax_warehouse WHERE 1=1 AND account = %%s AND token = %%s %s) b \
                    ON b.latest_rest_tx_no = t.tx_no \
                    UNION \
                    SELECT t.timestamp,b.latest_rest_tx_no,b.latest_rest_amount FROM {0}.tax_reward t RIGHT JOIN ( \
                    SELECT DISTINCT FIRST_VALUE(rest_tx_no) OVER (ORDER BY day DESC) as latest_rest_tx_no,FIRST_VALUE(rest_amount) OVER (ORDER BY day desc) as latest_rest_amount FROM {0}.tax_warehouse WHERE 1=1 AND account = %%s AND token = %%s %s) b \
                    ON b.latest_rest_tx_no = t.tx_no" % (conditions,conditions)).format(sql.Identifier(schema))

    result = herokuDbAccess.fetchDataInDatabase(query, [account,token,*params,account,token,*params], connection)  
    if len(result) == 0 or result[0][0] is None:
        return (0,None,None)
        #return datetime(1970, 1, 1, 0, 0, 0, tzinfo=pytz.timezone('UTC'))
        
    #d = result[0][0]
    return result[0]
    #return datetime(d.year,d.month,d.day, 0, 0, 0, tzinfo=pytz.timezone('UTC'))

def getLatestWarehouseDate(connection, schema, token:str, account:str, chain=None, chain_id=None):
    conditions = ""
    params = []
    if chain is None and chain_id is None:
        return []
    elif chain is not None:
        conditions = conditions + " AND chain = %s"
        params.append(chain)
    elif chain_id is not None:
        conditions = conditions + " AND chain_id = %s"
        params.append(chain_id)

    query = sql.SQL("SELECT MAX(day) as latest_day FROM {0}.tax_warehouse WHERE 1=1 AND account = %%s AND token = %%s %s" % (conditions)).format(sql.Identifier(schema))

    result = herokuDbAccess.fetchDataInDatabase(query, [account,token,*params], connection)  
    if result[0][0] is None:
        return datetime(1970, 1, 1, 0, 0, 0, tzinfo=pytz.timezone('UTC'))
        
    d = result[0][0]
    return datetime(d.year,d.month,d.day, 0, 0, 0, tzinfo=pytz.timezone('UTC'))


def postTaxReward(connection, schema, data):
    query = sql.SQL("INSERT INTO {}.tax_reward (tx_no,chain_id,chain,public_address,timestamp,latest_block_number,category,token,amount,usd_value,eur_value) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)").format(sql.Identifier(schema))   
    params = (
        data['tx_no'],
        data['chain_id'],
        data['chain'],
        data['public_address'],
        data['timestamp'],
        data['latest_block_number'],
        data['category'],
        data['token'],
        data['amount'],
        data['usd_value'],
        data['eur_value']
    )

    result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
    return result

def postTaxWarehouse(connection, schema, data):
    query = sql.SQL("INSERT INTO {}.tax_warehouse (chain_id,chain,account,day,token,amount,amount_usd,amount_eur,amount_in,amount_in_usd,amount_in_eur,amount_out,amount_out_usd,amount_out_eur,profit_amount_usd,profit_amount_eur,fee_amount_usd,fee_amount_eur,rest_tx_no,rest_amount) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)").format(sql.Identifier(schema))   
    params = (
        data['chain_id'],
        data['chain'],
        data['account'],
        data['day'],
        data['token'],
        data['amount'],
        data['amount_usd'],
        data['amount_eur'],
        data['amount_in'],
        data['amount_in_usd'],
        data['amount_in_eur'],
        data['amount_out'],
        data['amount_out_usd'],
        data['amount_out_eur'],
        data['profit_amount_usd'],
        data['profit_amount_eur'],
        data['fee_amount_usd'],
        data['fee_amount_eur'],
        data['rest_tx_no'],
        data['rest_amount']
    )

    result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
    return result

def postTaxTrade(connection, schema, data):
    query = sql.SQL("INSERT INTO {}.tax_trades (chain_id,chain,account,address,token,buy_timestamp,buy_price,buy_transaction_hashes,sell_timestamp,sell_price,sell_transaction_hashes,amount,fee_amount_usd,fee_amount_eur) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)").format(sql.Identifier(schema))   
    params = (
        data['chain_id'],
        data['chain'],
        data['account'],
        data['address'],
        data['token'],
        data['buy_timestamp'],
        data['buy_price'],
        data['buy_transaction_hashes'],
        data['sell_timestamp'],
        data['sell_price'],
        data['sell_transaction_hashes'],
        data['amount'],
        data['fee_amount_usd'],
        data['fee_amount_eur']
    )

    result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
    return result



# import os
# from dotenv import load_dotenv
# import herokuDbAccess as db
# from datetime import datetime
# load_dotenv()

# if __name__ == '__main__':
#     connection = db.connect(
#         os.environ.get("HEROKU_DB_USER"),
#         os.environ.get("HEROKU_DB_PW"),
#         os.environ.get("HEROKU_DB_HOST"),
#         os.environ.get("HEROKU_DB_PORT"),
#         os.environ.get("HEROKU_DB_DATABASE")
#     )
    # data = {
    #     'chain_id':1,
    #     'chain':'ETH',
    #     'account':'MYACC',
    #     'day':datetime.strptime('2022-12-01','%Y-%m-%d'),
    #     'token':'BLUE1',
    #     'amount':12.2,
    #     'amount_usd':120.1,
    #     'amount_eur':110.23,
    #     'amount_in':1.3,
    #     'amount_in_usd':1.4,
    #     'amount_in_eur':1.5,
    #     'amount_out':1.6,
    #     'amount_out_usd':1.7,
    #     'amount_out_eur':1.8,
    #     'tax_amount_usd':1.9,
    #     'tax_amount_eur':2.0
    # }
    # data = {
    #     'chain_id':1,
    #     'chain':'ETH',
    #     'account':'MYACC',
    #     'address':'dfi1',
    #     'token':'BLUE1',
    #     'buy_timestamp':127897934,
    #     'buy_price':12.4,
    #     'buy_transaction_hashes':'sjdlfjskljdfklsj,jhghjhghj',
    #     'sell_timestamp':893434,
    #     'sell_price':34.35,
    #     'sell_transaction_hashes':'shdsdfjkhsdjkf2,sdjfklj',
    #     'amount':12.2
    # }
    #res = getTaxTransactionRewards(connection,'stage',['dfi1','df1qgssyvcqld7cwxzvnajyt3xlqae67muxrqh06ct'],'DeFiChain',None,'DFI',0)
    #res = getTaxTokens(connection,'stage',['dfi1','df1qgssyvcqld7cwxzvnajyt3xlqae67muxrqh06ct'],'DeFiChain',None)
    # res = getLatestWarehouseDate(connection,'stage','BLUE12','MYACC','Defi',None)
    # print(res)


from psycopg2 import sql
from ..DataStructures import ObjEvmToken
from . import herokuDbAccess

def postEvmToken(connection, schema:str, chain_id, liquidity_pool_address, token_address_a, token_address_b, exchange_name):

    query = sql.SQL("INSERT INTO {}.evm_token (chain_id, liquidity_pool_address, token_address_a, token_address_b, exchange_name) \
           VALUES (%s,%s,%s,%s,%s)").format(sql.Identifier(schema))
    params = (
        chain_id,
        liquidity_pool_address,
        token_address_a,
        token_address_b,
        exchange_name)
    result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
    return result

def getEvmLiquidityPool(connection, schema, chain_id, token_address):
    
    query = sql.SQL("SELECT chain_id, liquidity_pool_address, token_address_a, token_address_b, exchange_name, created_at, modified_at \
                        FROM {}.evm_token_liquidity_pools \
                        WHERE chain_id = %s AND (token_address_a = %s OR token_address_b = %s").format(sql.Identifier(schema))
    
    result = herokuDbAccess.fetchDataInDatabase(query, [chain_id,token_address,token_address], connection)    
    return result



from unicodedata import decimal


class ObjViewEvmTokenLiqPool:
    token_address = None
    chain_id = None 
    abi = None
    symbol = None
    name = None
    decimals = 0
    token_class = ''
    totalsupply = 0
    keywords = ''
    telegram_link = ''
    creator_address = ''
    creation_timestamp = 0 
    creation_block_number = 0 
    creation_tx_hash = '' 
    max_tx_amount_percent = 0 
    max_wallet_size_percent = 0 
    is_verified = False 
    is_honeypot = False 
    buy_tax = 0 
    sell_tax = 0  
    latest_timestamp_token_history = 0 
    liquidity_pool_address = ''
    exchange_name = ''
    latest_timestamp_lp = 0
    created_at_lp = 0
    modified_at_lp = 0
    token_side = ''
    reserve_a = 0
    reserve_b = 0
    is_sellable = ''
    holder_count = 0
    volume_daily = 0

    def __init__(self):
        pass

    def __eq__(self, other): 
        if not isinstance(other, ObjViewEvmTokenLiqPool):
            # don't attempt to compare against unrelated types
            return NotImplemented

        return (self.token_address == other.token_address and
                self.chain_id == other.chain_id and
                self.abi == other.abi and
                self.symbol == other.symbol and
                self.name == other.name and
                self.decimals == other.decimals and
                self.token_class == other.token_class and
                self.totalsupply == other.totalsupply and
                self.keywords == other.keywords and
                self.telegram_link == other.telegram_link and
                self.creator_address == other.creator_address and
                self.creation_timestamp == other.creation_timestamp and
                self.creation_block_number == other.creation_block_number and
                self.creation_tx_hash == other.creation_tx_hash and
                self.max_tx_amount_percent == other.max_tx_amount_percent and
                self.max_wallet_size_percent == other.max_wallet_size_percent and
                self.is_verified == other.is_verified and
                self.is_honeypot == other.is_honeypot and
                self.buy_tax == other.buy_tax and
                self.sell_tax == other.sell_tax and
                self.latest_timestamp_token_history == other.latest_timestamp_token_history and
                self.liquidity_pool_address == other.liquidity_pool_address and
                self.exchange_name == other.exchange_name and
                self.latest_timestamp_lp == other.latest_timestamp_lp and
                self.created_at_lp == other.created_at_lp and
                self.modified_at_lp == other.modified_at_lp and
                self.token_side == other.token_side and
                self.reserve_a == other.reserve_a and
                self.reserve_b == other.reserve_b and
                self.is_sellable == other.is_sellable and
                self.holder_count == other.holder_count and
                self.volume_daily == other.volume_daily)

    def clone(self):
        retTok = ObjViewEvmTokenLiqPool()
        retTok.token_address = self.token_address
        retTok.chain_id = self.chain_id
        retTok.abi = self.abi
        retTok.symbol = self.symbol
        retTok.name = self.name
        retTok.decimals = self.decimals
        retTok.token_class = self.token_class
        retTok.totalsupply = self.totalsupply
        retTok.keywords = self.keywords
        retTok.telegram_link = self.telegram_link
        retTok.creator_address = self.creator_address
        retTok.creation_timestamp = self.creation_timestamp
        retTok.creation_block_number = self.creation_block_number
        retTok.creation_tx_hash = self.creation_tx_hash
        retTok.max_tx_amount_percent = self.max_tx_amount_percent
        retTok.max_wallet_size_percent = self.max_wallet_size_percent
        retTok.is_verified = self.is_verified
        retTok.is_honeypot = self.is_honeypot
        retTok.buy_tax = self.buy_tax
        retTok.sell_tax = self.sell_tax
        retTok.latest_timestamp_token_history = self.latest_timestamp_token_history
        retTok.liquidity_pool_address = self.liquidity_pool_address
        retTok.exchange_name = self.exchange_name
        retTok.latest_timestamp_lp = self.latest_timestamp_lp
        retTok.created_at_lp = self.created_at_lp
        retTok.modified_at_lp = self.modified_at_lp
        retTok.token_side = self.token_side
        retTok.reserve_a = self.reserve_a
        retTok.reserve_b = self.reserve_b
        retTok.is_sellable = self.is_sellable
        retTok.holder_count = self.holder_count
        retTok.volume_daily = self.volume_daily
        return retTok